# -*- coding: utf-8 -*-

DISPLAY_MENU = "display_menu"
DISPLAY_COLLECTION_MENU = "display_collection_menu"
DISPLAY_BACKENDS = "display_backends"

SEARCH = "search"
VIDEO = "video"
DOWNLOAD = "download"
UPDATE = "update"
