set WEBOOB_VERSION=1.3
set WEBOOB=weboob-1.3-py2.7.egg
set LIST_APPLIQUATIONS_QT=qboobmsg qcineoob qcookboob qflatboob qhandjoob qhavedate qvideoob qwebcontentedit weboob-config-qt qbooblyrics
REM x32 | x64
set ARCHITECTURE=x64
set LOCAL_ARCHITECTURE=x64
REM version required for WEBOOB
set PYTHON_VERSION=2.7
set PYTHON_MINOR_VERSION=6
